/*------------------------------------------------------*/
/*		Module:				int.c						*/
/*		writen by:			Michael Klose				*/
/*		date:				27.04.2005					*/
/*		last modified by:	Michael Klose				*/
/*		date:				27.04.2005					*/
/*		functions related to trigger-interrupts			*/
/*------------------------------------------------------*/

#include "main.h"

//declaration of functions

//declaration of variables
extern volatile unsigned char counter;					 	//declared in main.c
extern volatile unsigned char time1, time2, time3, phase;	//declared in main.c
extern volatile bit usePhase;								//declared in main.c
extern volatile bit mainc;									//declared in main.c

/*------------------------------------------------------*/
/*	function InterruptInit								*/
/*	initializes the trigger-interrupts					*/
/*	no input, no return value							*/
/*	shoot -> EXINT2	(both edges)									
	trigger -> EXINT3 (rising edge)
	phase -> EXINT1 (rising edge) NOT ACTIVATED!!!		*/

void InterruptInit(void)
{
	//set priorities of interrupts (shoot = 2, trigger and phase = 1)
	IP1 = 0x8;							//reset bit for shoot, set bit for trigger
	IPH1 = 0x4;							//set bit for shoot, reset bit for trigger
	IP |= 0x4;							//set bit for phase
	IPH = 0x3b;							//reset bit for phase

	EXICON0 = 0x14;						//edge selection for the three signals
	IEN0 &= 0xfb;						//disable interrupt for phase (individually)
	IEN1 |= 0x4;						//enable interrupt for the shoot(individually)
	IEN1 |= 0x8;						//enable interrupt for trigger  (individually)
}
//end of function InterruptInit()

/*------------------------------------------------------*/
/*	function TriggerISR									*/
/*	ISR for trigger signal to latch time and phase		*/
/*	no input, no return value							*/

void TriggerISR() interrupt 8
{
	IRCON0 &= 0xfb;					//reset IR-Bit of trigger

	switch (counter)
	{
		case 0:
			time1 = P3_DATA; 		//get LSB of time
			if (time1 == 0)
			{
				P0_DATA &= 0xf8;
				P0_DATA |= 1;
			}
			else
			{
				if (time1 == 1)
				{
					P0_DATA &= 0xf8;
					P0_DATA |= 2;
				}
			}
			break;
		
		case 1:
			time2 = P3_DATA;		//get 2nd byte of time
			if (time2 == 0)
			{
				P0_DATA &= 0xf8;
				P0_DATA |= 0x3;
			}
			else
			{
				if (time2 == 1)
				{
					P0_DATA &= 0xf8;
					P0_DATA |= 0x4;
				}
			}
			break;

		case 2:
			time3 = P3_DATA;	  	//get MSB of time
			if (time3 == 0)
			{
				P0_DATA &= 0xf8;
				P0_DATA |= 0x1;
			}
			else
			{
				if (time3 == 13)
				{
					P0_DATA &= 0xf8;
					P0_DATA |= 0x7;
				}
			}
			break;

		case 3:
			phase = P3_DATA;		//get phase information
			if (phase == 0)
			{
				P0_DATA &= 0xf8;
				P0_DATA |= 0x6;
			}
			else
			{
				if (phase == 128)
				{
					P0_DATA &= 0xf8;
					P0_DATA |= 0x1;
				}
			}
			break;

		case 4:
			switch (P3_DATA)
			{
				case 0:
					usePhase = 0;		  	//-> phase ignored
					mainc = 0;				//use serial contactors
					break;
				case 1:
					usePhase = 1;			//-> phase important
					mainc = 0;				//use serial contactors
					break;
				case 2:
					usePhase = 0;	 		//-> phase ignored
					mainc = 1;				//use main contactor
					break;
				case 3:
					usePhase = 1;			//-> phase important
					mainc = 1;				//use main contactor
					break;
			}
			break;
	}
	counter++;
}
//end of function TriggerISR()
